@@ -1,4 +1,8 @@
-"""Define a Request class and child classes.
+"""
+aspen.http.request
+~~~~~~~~~~~~~~~~~~
+
+Define a Request class and child classes.
 
 Here is how we analyze the structure of an HTTP message, along with the objects
 we use to model each:
@@ -8,6 +12,7 @@ we use to model each:
             - method            Method      ASCII
             - uri               URI
                 - path          Path
+                  - parts       list of PathPart
                 - querystring   Querystring
             - version           Version     ASCII
         - headers               Headers     str
@@ -25,8 +30,13 @@ XXX TODO
     clean up body
 
 """
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
+
+
 import cgi
-import mimetypes
 import re
 import sys
 import urllib
@@ -36,8 +46,7 @@ from cStringIO import StringIO
 from aspen import Response
 from aspen.http.baseheaders import BaseHeaders
 from aspen.http.mapping import Mapping
-from aspen.context import Context
-from aspen.utils import ascii_dammit, typecheck
+from aspen.utils import ascii_dammit
 
 
 # WSGI Do Our Best
@@ -77,7 +86,7 @@ def make_franken_uri(path, qs):
             # Some servers (gevent) clobber %2F inside of paths, such
             # that we see /foo%2Fbar/ as /foo/bar/. The %2F is lost to us.
             parts = [urllib.quote(x) for x in quoted_slash_re.split(path)]
-            path = "%2F".join(parts)
+            path = b"%2F".join(parts)
 
     if qs:
         try:
@@ -87,7 +96,7 @@ def make_franken_uri(path, qs):
             # perform the percent-encoding that we would expect MSIE to have
             # done for us.
             qs = urllib.quote_plus(qs)
-        qs = '?' + qs
+        qs = b'?' + qs
 
     return path + qs
 
@@ -98,31 +107,31 @@ def make_franken_headers(environ):
 
     # There are a couple keys that CherryPyWSGIServer explicitly doesn't
     # include as HTTP_ keys. I'm not sure why, but I believe we want them.
-    also = ['CONTENT_TYPE', 'CONTENT_LENGTH']
+    also = [b'CONTENT_TYPE', b'CONTENT_LENGTH']
 
     headers = []
     for k, v in environ.items():
         val = None
-        if k.startswith('HTTP_'):
-            k = k[len('HTTP_'):]
+        if k.startswith(b'HTTP_'):
+            k = k[len(b'HTTP_'):]
             val = v
         elif k in also:
             val = v
         if val is not None:
-            k = k.replace('_', '-')
-            headers.append(': '.join([k, v]))
+            k = k.replace(b'_', b'-')
+            headers.append(b': '.join([k, v]))
 
-    return '\r\n'.join(headers)  # *sigh*
+    return b'\r\n'.join(headers)
 
 
 def kick_against_goad(environ):
     """Kick against the goad. Try to squeeze blood from a stone. Do our best.
     """
     method = environ['REQUEST_METHOD']
-    uri = make_franken_uri( environ.get('PATH_INFO', '')
-                          , environ.get('QUERY_STRING', '')
-                           )
-    server = environ.get('SERVER_SOFTWARE', '')
+    uri = make_franken_uri( environ.get('PATH_INFO', b'')
+                          , environ.get('QUERY_STRING', b'')
+                          )
+    server = environ.get('SERVER_SOFTWARE', b'')
     version = environ['SERVER_PROTOCOL']
     headers = make_franken_headers(environ)
     body = environ['wsgi.input']
@@ -157,6 +166,16 @@ class UnicodeWithRaw(unicode):
         obj.raw = raw
         return obj
 
+class PathPart(unicode):
+    """A string with a mapping for extra data about it."""
+
+    __slots__ = ['params']
+
+    def __new__(cls, value, params):
+        obj = super(PathPart, cls).__new__(cls, value)
+        obj.params = params
+        return obj
+
 
 ###########
 # Request #
@@ -166,7 +185,6 @@ class Request(str):
     """Represent an HTTP Request message. It's bytes, dammit. But lazy.
     """
 
-    socket = None
     resource = None
     original_resource = None
     server_software = ''
@@ -176,9 +194,9 @@ class Request(str):
     #   http://docs.python.org/reference/datamodel.html#__slots__
 
 
-    def __new__(cls, method='GET', uri='/', server_software='',
-                version='HTTP/1.1', headers='', body=None):
-        """Takes five bytestrings and an iterable of bytestrings.
+    def __new__(cls, method=b'GET', uri=b'/', server_software=b'',
+                version=b'HTTP/1.1', headers=b'', body=None):
+        """Takes five bytestrings and a file-like object.
         """
         obj = str.__new__(cls, '') # start with an empty string, see below for
                                    # laziness
@@ -186,17 +204,14 @@ class Request(str):
         try:
             obj.line = Line(method, uri, version)
             if not headers:
-                headers = 'Host: localhost'
+                headers = b'Host: localhost'
             obj.headers = Headers(headers)
             if body is None:
                 body = StringIO('')
-            obj.body = Body( obj.headers
-                           , body
-                           , obj.server_software
-                            )
-            obj.context = Context(obj)
+            raw_len = int(obj.headers.get('Content-length', '') or '0')
+            obj.raw_body = body.read(raw_len)
+            obj.context = {}
         except UnicodeError:
-
             # Figure out where the error occurred.
             # ====================================
             # This gives us *something* to go on when we have a Request we
@@ -214,6 +229,25 @@ class Request(str):
         return obj
 
 
+    @property
+    def body(self):
+        '''Lazily parse the body, iff _parse_body is set.
+           Otherwise default to raw_body.  In the normal course of things,
+           _parse_body is set in aspen.algorithm.website
+        '''
+        if hasattr(self, 'parsed_body'):
+            return self.parsed_body
+        if hasattr(self, '_parse_body'):
+            self.parsed_body = self._parse_body(self)
+            return self.parsed_body
+        return self.raw_body
+
+    @body.setter
+    def body(self, value):
+        '''Let the developer set the body to something if they want'''
+        self.parsed_body = value
+
+
     @classmethod
     def from_wsgi(cls, environ):
         """Given a WSGI environ, return an instance of cls.
@@ -241,7 +275,7 @@ class Request(str):
         """
         if not self._raw:
             fmt = "%s\r\n%s\r\n\r\n%s"
-            self._raw = fmt % (self.line.raw, self.headers.raw, self.body.raw)
+            self._raw = fmt % (self.line.raw, self.headers.raw, self.raw_body)
         return self._raw
 
     def __repr__(self):
@@ -295,21 +329,6 @@ class Request(str):
         raise Response(code, headers={'Location': location})
 
 
-    def _infer_media_type(self):
-        """Guess a media type based on our filesystem path.
-
-        The gauntlet function indirect_negotiation modifies the filesystem
-        path, and we want to infer a media type from the path before that
-        change. However, we're not ready at that point to infer a media type
-        for *all* requests. So we need to perform this inference in a couple
-        places, and hence it's factored out here.
-
-        """
-        media_type = mimetypes.guess_type(self.fs, strict=False)[0]
-        if media_type is None:
-            media_type = self.website.media_type_default
-        return media_type
-
 # Request -> Line
 # ---------------
 
@@ -459,6 +478,35 @@ class URI(unicode):
         obj.raw = raw
         return obj
 
+def extract_rfc2396_params(path):
+    """RFC2396 section 3.3 says that path components of a URI can have
+    'a sequence of parameters, indicated by the semicolon ";" character.'
+    and that ' Within a path segment, the characters "/", ";", "=", and
+    "?" are reserved.'  This way you can do
+    /frisbee;color=red;size=small/logo;sponsor=w3c;color=black/image.jpg
+    and each path segment gets its own params.
+
+    * path should be raw so we don't split or operate on a decoded character
+    * output is decoded
+    """
+    pathsegs = path.lstrip(b'/').split(b'/')
+    def decode(input):
+        return urllib.unquote(input).decode('UTF-8')
+
+    segments_with_params = []
+    for component in pathsegs:
+        parts = component.split(b';')
+        params = Mapping()
+        segment = decode(parts[0])
+        for p in parts[1:]:
+            if '=' in p:
+                k, v = p.split(b'=', 1)
+            else:
+                k, v = p, b''
+            params.add(decode(k), decode(v))
+        segments_with_params.append(PathPart(segment, params))
+    return segments_with_params
+
 
 # Request -> Line -> URI -> Path
 
@@ -470,8 +518,9 @@ class Path(Mapping):
     """
 
     def __init__(self, raw):
-        self.decoded = urllib.unquote(raw).decode('UTF-8')
         self.raw = raw
+        self.decoded = urllib.unquote(raw).decode('UTF-8')
+        self.parts = extract_rfc2396_params(raw)
 
 
 # Request -> Line -> URI -> Querystring
@@ -568,129 +617,3 @@ class Headers(BaseHeaders):
 
         scheme = 'https' if self.get('HTTPS', False) else 'http'
         self.scheme = UnicodeWithRaw(scheme)
-
-
-# Request -> Body
-# ---------------
-
-class Body(Mapping):
-    """Represent the body of an HTTP request.
-    """
-
-    def __init__(self, headers, fp, server_software):
-        """Takes a str, a file-like object, and another str.
-
-        If the Mapping API is used (in/one/all/has), then the iterable will be
-        read and parsed as media of type application/x-www-form-urlencoded or
-        multipart/form-data, according to content_type.
-
-        """
-        typecheck(headers, Headers, server_software, str)
-        self.raw = self._read_raw(server_software, fp)  # XXX lazy!
-        parsed = self._parse(headers, self.raw)
-        if parsed is None:
-            # There was no content-type. Use self.raw.
-            pass
-        else:
-            for k in parsed.keys():
-                v = parsed[k]
-                if isinstance(v, cgi.MiniFieldStorage):
-                    v = v.value.decode("UTF-8")  # XXX Really? Always UTF-8?
-                else:
-                    assert isinstance(v, cgi.FieldStorage), v
-                    if v.filename is None:
-                        v = v.value.decode("UTF-8")
-                self[k] = v
-
-
-    def _read_raw(self, server_software, fp):
-        """Given str and a file-like object, return a bytestring.
-        """
-        if not server_software.startswith('Rocket'):  # normal
-            raw = fp.read()
-        else:                                                       # rocket
-
-            # Email from Rocket guy: While HTTP stipulates that you shouldn't
-            # read a socket unless you are specifically expecting data to be
-            # there, WSGI allows (but doesn't require) for that
-            # (http://www.python.org/dev/peps/pep-3333/#id25).  I've started
-            # support for this (if you feel like reading the code, its in the
-            # connection class) but this feature is not yet production ready
-            # (it works but it way too slow on cPython).
-            #
-            # The hacky solution is to grab the socket from the stream and
-            # manually set the timeout to 0 and set it back when you get your
-            # data (or not).
-            #
-            # If you're curious, those HTTP conditions are (it's better to do
-            # this anyway to avoid unnecessary and slow OS calls):
-            # - You can assume that there will be content in the body if the
-            #   request type is "POST" or "PUT"
-            # - You can figure how much to read by the "CONTENT_LENGTH" header
-            #   existence with a valid integer value
-            #   - alternatively "CONTENT_TYPE" can be set with no length and
-            #     you can read based on the body content ("content-encoding" =
-            #     "chunked" is a good example).
-            #
-            # Here's the "hacky solution":
-
-            _tmp = fp._sock.timeout
-            fp._sock.settimeout(0) # equiv. to non-blocking
-            try:
-                raw = fp.read()
-            except Exception, exc:
-                if exc.errno != 35:
-                    raise
-                raw = ""
-            fp._sock.settimeout(_tmp)
-
-        return raw
-
-
-    def _parse(self, headers, raw):
-        """Takes a dict and a bytestring.
-
-        http://www.w3.org/TR/html401/interact/forms.html#h-17.13.4
-
-        """
-        typecheck(headers, Headers, raw, str)
-
-
-        # Switch on content type.
-
-        parts = [p.strip() for p in headers.get("Content-Type", "").split(';')]
-        content_type = parts.pop(0)
-
-        # XXX Do something with charset.
-        params = {}
-        for part in parts:
-            if '=' in part:
-                key, val = part.split('=', 1)
-                params[key] = val
-
-        if content_type == "application/x-www-form-urlencoded":
-            # Decode.
-            pass
-        elif content_type == "multipart/form-data":
-            # Deal with bytes.
-            pass
-        else:
-            # Bail.
-            return None
-
-
-        # Force the cgi module to parse as we want. If it doesn't find
-        # something besides GET or HEAD here then it ignores the fp
-        # argument and instead uses environ['QUERY_STRING'] or even
-        # sys.stdin(!). We want it to parse request bodies even if the
-        # method is GET (we already parsed the querystring elsewhere).
-
-        environ = {"REQUEST_METHOD": "POST"}
-
-
-        return cgi.FieldStorage( fp = cgi.StringIO(raw)  # Ack.
-                               , environ = environ
-                               , headers = headers
-                               , keep_blank_values = True
-                               , strict_parsing = True
-                                )
